/**********************************************************************
 * 			MEKATRONIX Copyright 1998 		      *
 * Title        servotjp.c  (Servo Drivers for TJ PRO)                *
 * Programmer   Keith L. Doty			                      *
 * Date         September 15, 1998                                    *
 * Version      1                                                     *
 *                                                                    *
 * Description: Servo drivers based on the TJ standard drivers        *
 * History:     Based on earlier versions by Greene, Doty, Bagnell.   *
 **********************************************************************/


/**************************** Includes **********************************/
#include <hc11.h>
#include <mil.h>
/************************************************************************/


/**************************** Constants *********************************/
#define HALFPERIOD 20000         /* 50Hz refresh for each servo */
#define servo_hand  TOC4_isr
/************************************************************************/


/*************************** Prototypes *********************************/
#pragma interrupt_handler TOC4_isr    /*Servo Handler*/
void init_servotjp(void);

/* servo() arguments: Servo# (0,1,2), Pulse Width(0 or 2000 to 4000) */
void servo(int, int);

void servo_hand();
/************************************************************************/


/***************************** Globals **********************************/
/* servo_width[] is the on-time for each servo*/
unsigned int servo_width[3]; 
unsigned int current_servo_width;
char servo_signal_state, servomask[3];
/************************************************************************/



void init_servotjp(void)
/************************************************************************
 * Function: Initializes all necessary variables for the servos.        *
 * Returns: None                                                        *
 *                                                                      *
 * Inputs                                                               *
 *   Parameters: None                                                   *
 *   Globals:    None                                                   *
 *   Registers:  None                                                   *
 * Outputs                                                              *
 *   Parameters: None                                                   *
 *   Globals:   servo_width[], servo_signal_state, servomask[],		*
 *              current_servo_width       					*
 *   Registers: TOC1, PORTB, TMSK1, TCTL1                               *
 * Functions called: None                                               *
 * Notes: None                                                          *
 ************************************************************************/
{

  INTR_OFF();
  
  /*install interrupt handler on OC4*/
  *((void (**)())0xffe2)  = servo_hand;
  
  /* Motors start turned off */
  servo_width[0] = servo_width[1] = servo_width[2] =  0; 
  TOC4 = 0;                       
  servo_signal_state = 0;
  current_servo_width = 0;
  servomask[0] = 0x10;              /* modified servo masks:PA4, OC4*/
  servomask[1] = 0x20;              /* PA5, OC3*/
  servomask[2] = 0x40;              /* PA6, OC2*/

  SET_BIT(TMSK1,0x10);              /* Enable OC4 interrupt */
  INTR_ON();
}


void servo(int index, int newwidth)
/************************************************************************
 * Function: Sets one of the servos to the speed defined by newwidth    *
 * Returns: None                                                        *
 *                                                                      *
 * Inputs                                                               *
 *   Parameters: index, newwidth                                        *
 *   Globals:    None                                                   *
 *   Registers:  None                                                   *
 * Outputs                                                              *
 *   Parameters: None                                                   *
 *   Globals: servo_width[index]                                              *
 *   Registers:  None                                                   *
 * Functions called: None                                               *
 * Notes: None                                                          *
 ************************************************************************/
{
  asm("ldd %index \n"
      "aslb \n"
      "ldy #_servo_width \n"
      "aby \n"
      "ldd %newwidth \n"
      "std 0,y");
}



void servo_hand(void)
/************************************************************************
 * Function: Interrupt handler for servo signals.                       *
 *	     Interrupt is OC4. Servo_0 is on PA4 and Servo_1 on PA5 and 	*
 *           Servo_2 is on PA6.							*
 * Returns: None                                                        *
 *                                                                      *
 * Inputs                                                               *
 *   Parameters: None                                                   *
 *   Globals:    servo_width[], servo_signal_state, servomask[], 		*
 *               current_servo_width      					*
 *   Registers:  None                                                   *
 * Outputs                                                              *
 *   Parameters: None                                                   *
 *   Globals:    current_servo_width                                    *
 *   Registers:  TOC4, PORTA, TFLG1                                     *
 * Functions called: None                                               *
 * Notes: None                                                          *
 ************************************************************************/
{
  char odd, index;
  unsigned int pwidth;

/*
   servo_signal_state = 0  -> Turn on servo0
   servo_signal_state = 1  -> Turn off servo0
   servo_signal_state = 2  -> Turn on servo1
   servo_signal_state = 3  -> Turn off servo1
   servo_signal_state = 4  -> Turn on servo2
   servo_signal_state = 5  -> Turn off servo2

*/

  servo_signal_state &= 0x07;         /* Only use last 3 bits */
  index = (servo_signal_state >> 1);  /* index references current servo */

  odd = (servo_signal_state & 0x01);

  pwidth = *(servo_width+index);
  
  if ((pwidth == 0)&&(!(odd)))
  {
    TOC4 += HALFPERIOD;
    servo_signal_state++;
  }
  else
  {
    if (odd)
      TOC4 += (HALFPERIOD - current_servo_width);
    else
      TOC4 += pwidth;
  
    PORTA ^= *(servomask+index);
  }

  current_servo_width = pwidth;
  servo_signal_state++;
  if (servo_signal_state == 6) servo_signal_state = 0;
  
   CLEAR_FLAG(TFLG1,0x10);          /* Clear OC4I flag */
}
