/************************************************************************
 * 			MEKATRONIX Copyright 1998 		        *
 * Title        avsrvtjp.c                                              *
 * Programmer	Keith L. Doty and modified for IC by Scott Jantz     	*
 * Date         August 25, 1998                                         *
 * Version	1			                		*
 *								        *
 * Description								*
 *    A very simple collision avoidance program. TJ PRO will read 	*
 *    each IR detectors, and turn away from any obstacles in its	*
 *    path. Also, if something hits TJ PRO's bumper, it will back up,	*
 *    turn, and go on. A servo, connected to Servo_1 (PA5), when mounted*
 *    on top of the TJ Pro, will turn its horn in the direction of the  *
 *    bump. If a sensor platform is mounted on the horn, TJ Pro		*
 *    essentially turns its head towards the point of contact on the	*
 *    bumper. This motion seems organic and alive!			*

Switch Closure(s)	Range		Turn Servo to Angle(nominal)
Front-Middle		40-50	90 degrees  (pw_count=2700)
Front-Right		20-30	45 degrees  (pw_count=1900)
Front-Left		67-70	135 degrees (pw_count=3500)
Front-Middle:Front-Right60-63	68 degrees  (pw_count=2300)
Front-Middle:Front-Left	90-100	113 degrees (pw_count=3100)
Back			120-135	90 degrees  (pw_count=2700)

After you calibrate your servo you may want to change the nominal
values for the pw_count to values dictated by your calibration.
 *			                                                *     					*
 ************************************************************************/
 

/*************************** Includes ********************************/

#include <tjpbase.h>
#include <stdio.h>

/************************ End of Includes ****************************/


/*************************** Constants ********************************/

#define AVOID_THRESHOLD 100

/************************ End of Constants ****************************/


/*************************** Prototypes *********************************/
void turn(void);
/************************ End of Prototypes *****************************/

void main()
/****************************** Main ***********************************/
{
  int bump, irdr, irdl, pw_count, speedr, speedl;

  init_analog();
  init_motortjp();
  init_clocktjp();
  init_servotjp();

  IRE_ON;     /* turn on IR emitters */


  START;  /*Press the rear bumper to start the program*/
  
   while(1)
  {

/*
   The following block will read the IR detectors, and decide whether TJ
   needs to turn to avoid any obstacles
*/
    irdr = RIGHT_IR;
    irdl = LEFT_IR;

    if (irdl > AVOID_THRESHOLD)
      speedr = -MAX_SPEED;
    else
      speedr = MAX_SPEED;
    if (irdr > AVOID_THRESHOLD)
      speedl = -MAX_SPEED;
    else
      speedl = MAX_SPEED;

    motorp(RIGHT_MOTOR, speedr); 
    motorp(LEFT_MOTOR, speedl);

/*
   This "if" statement checks the bumper. If the bumper is pressed, the servo
   turns toward the switch that was bumped. Tj will, then, back up and turn away from
   the object bumped.  
*/
  bump = BUMPER;

/*Turn servo towards the bump */

  if((bump>19) && (bump<31))
    {
     pw_count = 1900;
    }

  if((bump>59) && (bump<64))
    {
      pw_count = 2300;
    }


  if((bump>119) && (bump<136))
    {
      pw_count = 3500;
    }

  if((bump>39) && (bump<51))
    {
      pw_count = 3300;
    }

  if((bump>66) && (bump<71))
    {
      pw_count = 4300;
    }

  if((bump>89) && (bump<101))
    {
      pw_count = 3400;
    }

  servo(1,pw_count);
  servo(2, pw_count);
  wait(10);

    if((10 <bump)&&(bump<120))
    {
      motorp(LEFT_MOTOR, -MAX_SPEED);
      motorp(RIGHT_MOTOR, -MAX_SPEED);
      wait(600);
      turn();
    }

    wait(35);
  }    
}
/**************************** End of Main ******************************/

void turn()
/************************************************************************
 * Function:  Will turn in a random direction for a "random" amount of  *
 *            time, dictated by the fast changine lower bits in		*
 *	      mseconds().  					        *
 * Returns:  None                                                       *
 *                                                                      *
 * Inputs                                                               *
 *   Parameters: None                                                   *
 *   Globals:    None                                                   *
 *   Registers:  TCNT                                                   *
 * Outputs                                                              *
 *   Parameters: None                                                   *
 *   Globals:    None                                                   *
 *   Registers:  None                                                   *
 * Functions called: motorp(), wait()					*
 * Notes:                                                               *
 ************************************************************************/
{
 int i;
 unsigned rand;

 rand = TCNT;

 if (rand & 0x0001)
 /*turn left*/
 {
   motorp(RIGHT_MOTOR, MAX_SPEED);
   motorp(LEFT_MOTOR, -MAX_SPEED);
 }
 else
 /*turn right*/
 {
   motorp(RIGHT_MOTOR, -MAX_SPEED);
   motorp(LEFT_MOTOR, MAX_SPEED);
 }

 i=(rand % 1024);
 if(i>250) wait(i); else wait(250);

}

/***********************End Function turn ****************************/


