/**********************************************************************
*                        MEKATRONIX Copyright 1998
*
* Title        tjpargos.c
* Programmers  Keith L. Doty
* Date         Sept. 09, 1998
* Version      1

* Description 
This program outputs the values of the IR detectors and the bumper switches
continuously. This program lets you test the basic sensor suite of the
TJ Pro robot.

A physical, serial communication link must be present, e.g. the standard
Mekatronix serial cable connection via the MB2325 board.

Serial Protocol of this program:
  baud=9600, 8 data bits, 1 stop bit, no parity, no flow control.
 
 Example: 
   Configure Hyperterm on Win95:
   Direct to COM1, 9600 baud, 8 bits, no parity, 1 stop bit,
   no flow control.

**********************************************************************/

/*************************** Includes ********************************/
#include <tjpbase.h>
#include <stdio.h>
/************************ End of Includes ****************************/

void sensors_tst(void);
void motor_tst(void);
void servo_tst(void);

/***************************** Main **********************************/
void main(void)
{
  unsigned int bl, cv, fb, rb, run_test;
  char clear[]= "\x1b\x5B\x32\x4A\x04";  /* clear screen */ 
  char place[]= "\x1b[1;1H";		/* Home cursor */

  init_analog();
  init_clocktjp();
  init_serial();
  init_motortjp();
  init_servotjp();


  printf("%s", clear);  /*clear screen*/
  printf("%s", place);  /*home cursor*/

  printf("\tTitle\t\ttjprotst.c\n"
  "\tProgrammer\tKeith L. Doty\n"
  "\tDate\t\tMarch 22, 1999\n"
  "\tVersion\t\t1\n\n");

  printf("Select one of the following TJ PRO test procedures\n"
         "   1. Test TJ PRO Sensors \n"
         "   2. Test TJ PRO Servos\n"
	     "   3. Test TJ PRO Motors(CAREFUL!!! The robot will start moving immediately!)\n\n");

  run_test= read_int();

    switch (run_test)/*Select a test procedure */
       {
         case 1: sensors_tst(); break;
         case 2: servo_tst(); break;
	     case 3: motor_tst(); break;
       }
  
}

/***************************** Sensor **********************************/
void sensors_tst(void)
{

/* VT100 clear screen */ 
  char clear[]= "\x1b\x5B\x32\x4A\x04"; 

/* VT100 position cursor at (x,y) = (3,12) command is "\x1b[3;12H"*/
  char place[]= "\x1b[1;1H";	/*Home*/	

  IRE_ON; /*turn on IR emitters */

  printf("%s", clear);  
  printf("%s", place);
  
  printf("\tTitle\t\tsensetjp.c\n"
  "\tProgrammer\tKeith L. Doty\n"
  "\tDate\t\tSeptember, 1998\n"
  "\tVersion\t\t1\n\n");

  printf("TALRIK JUNIOR PROFESSIONAL SENSOR READINGS\n\n");
  printf("Left IR    Right IR    Bumper\n"); 

  while(1)
  {

  /*
    Move cursor to data field position and print data
    The escape sequences control cursor position ESC[<x_pos>;<y_pos>H
  */
    printf("\x1b[10;1H    \b\b\b\b\b%d",LEFT_IR);
    printf("\x1b[10;13H   \b\b\b\b%d",RIGHT_IR);
    printf("\x1b[10;25H   \b\b\b\b%d",BUMPER);
    wait(50);

  }/*end while(1)*/
}
/***************************** End of Sensor **********************************/


void motor_tst(void)
/**************************** motor_tst *********************************
 * Description								*
 *   Input two integers to specify the speed and direction of the left  *
 *   and right wheels. The integers must lie between -100 and +100. Any *
 *   non-digit character will terminate the integer, e.g., a blank.     *
 *									*
 *   	      CAREFUL!!! The robot will start moving immediately!	*
 *									*
 *   To input another pair of speeds, press reset to start the program  *
 *   over. 								*
 *									*
 *  Usage: This function allows you to move the robot at any fixed      *
 *         wheel speed ratios to generate either straight or circular	*
 *         motion.  							*
 ************************************************************************/
{
  int i;
/* VT100 clear screen */ 
  char clear[]= "\x1b\x5B\x32\x4A\x04"; 

/* VT100 position cursor at (x,y) = (3,12) command is "\x1b[3;12H"*/
  char place[]= "\x1b[1;1H";	/*Home*/	

  printf("%s", clear);  
  printf("%s", place);
 
  printf("Motor Speed = \n\n");
  printf("Press TJ PRO Reset Switch to select another test.");

  while(1)
  {
    for( i=-100; i< 101; i +=10)
    {
      motorp(RIGHT_MOTOR, i); 
      motorp(LEFT_MOTOR, i);
      printf("\x1b[1;14H      ");
      printf("\x1b[1;14H%d", i);
      wait(1000);
    }

    for( i=100; i> -101; i -=10)
    {
      motorp(RIGHT_MOTOR, i); 
      motorp(LEFT_MOTOR, i);
      printf("\x1b[1;14H      ");
      printf("\x1b[1;14H%d", i);
      wait(1000);
    }
  }

}
/**************************** motor_tst ******************************/


void servo_tst(void)
{
  int i;
  
  printf("Press TJ PRO Reset Switch to select another test.");

  while(1)
    {  
      for(i=1000; i<4501; i+=20) /*Go in one direction*/
        {
          servo(0, i);  /*SERVOZ, PA4*/
          servo(1, i);  /*SERVO1, PA5*/
          servo(2,i);   /*SERVO2, PA6*/
          wait(40);     /*Wait 40 milliseconds between commands*/
        }
      for(i=4500; i>999; i-=20) /*Reverse direction*/
        {
          servo(0, i);
          servo(1, i);
          servo(2,i);
          wait(40);
        }
    }
 
}
/**************************** End of servo_tst **************************/

